/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESAction;
import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;

import java.util.Vector;
import java.util.StringTokenizer;

/**
 * Check parameters
 * Input parameters:
 * Mandatory (a list of mandatory params)  (MANDATORY)
 * All (OPTIONAL): if true, the list of all missing params is returned
 *  (comma-separated).
 * ErrorTemplate  (OPTIONAL)
 *  Examples of templates:
 *  "Please fill in mandatory parameters."
 *  "Parameter $MissingField$ is missing."
 * Output parameters upon success:
 * Output parameters upon error:
 * MissingField (the name of the 1st mandatory field that is missing)
 * Message (the error message)
 */
public class CheckParams implements ESAction {

  public boolean serverSide() { return false; }

  public boolean processRequest(Client session, ESServletRequest req,
  DynHtml page, Object info) throws Exception {

    String mandatory[];
    String mandp = req.getParameter("Mandatory");
    if(mandp == null) return true;

    StringTokenizer st = new StringTokenizer(mandp, ",");
    Vector v = new Vector();

    while(st.hasMoreTokens()) {
      v.addElement(st.nextToken());
    }

    mandatory = new String[v.size()];
    for(int i=0; i<v.size(); i++) {
      mandatory[i] = (String)v.elementAt(i);
    }

    boolean all = false;
    String allp = req.getParameter("All");
    if(allp != null)
      all = allp.equalsIgnoreCase("true");

    String undefined = req.checkParameters(mandatory, all);
    if(undefined != null) {
      req.addParameter("MissingField", undefined);
      String tmpl = req.getParameter("ErrorTemplate");
      if(tmpl == null)
       tmpl = "Parameter " + undefined + " should have a value.";
      String msg;
      if(tmpl.indexOf('$') < 0) {
        msg = tmpl;
      } else {
        DynHtml expander = new ESDynHtml(session, null);
        expander.setServletRequest(req);
        msg = expander.expandString(tmpl);
      }
      req.addParameter("Message", msg);

      return false;
    }

    return true;
  }

};

