/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.db;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.sql.Connection;

import org.experlog.db.ESDBDataTuple;
import org.experlog.openeas.api.DataSet;
import org.experlog.openeas.api.DataObject;
import org.experlog.util.Client;
import org.experlog.util.ESCategory;
import org.enhydra.jdbc.standard.StandardStatement;
import org.enhydra.jdbc.core.CoreStatement;
import org.apache.log4j.Category;

public class ESDBDataSet implements DataSet {

  String table_ = null;
  ResultSet rset_ = null;

  int low_ = 1;
  int high_ = -1;
  int idx_ = 0;

  boolean first_ = true;
  boolean hasresults_ = false;

  // Cache ResultSet columns... due to ODBC bugs (!)
  boolean cacheRS_ = false;
  Connection con_ = null;
  Client session_ = null;
  String datasourcename_ = null;
  Category logger_ = null;
  Statement stat_ = null;

  public ESDBDataSet(ResultSet rs, Client session, Connection con, String datasourcename, Statement stat) throws SQLException { 
    logger_ =
      ESCategory.getCategory((session == null ? null : session.getAppName()),
          "openeas.db");
    con_ = con;
    rset_ = rs;
    stat_ = stat;
    logger_.debug ("ESDBDataSet:RSET "+stat_.toString());
    session_ = session;
    datasourcename_ = datasourcename;

    ResultSetMetaData md = rset_.getMetaData();
    if (md != null) table_ = md.getTableName(1);
    try {
      hasresults_ = rset_.next();
    } catch (Exception e) {
      hasresults_ = false;
    }
    if (hasresults_) idx_ = 1;

    cacheRS_ = session_.cacheRS();
  }

  public boolean next() throws SQLException {
    logger_.debug("ESDBDataSet:next");
    if (!hasresults_) return false;

    if (high_ > 0 && idx_ > high_) {
      hasresults_ = false;
      return false;
    }

    if (first_) {
      first_ = false;
    } else {
      try {
        if (hasresults_) hasresults_ = rset_.next();
      } catch (Exception e) {
        hasresults_ = false;
      }
    }
    if (hasresults_) idx_++;
    return hasresults_;
  }

  public boolean dataAvailable() throws SQLException {
    logger_.debug("ESDBDataSet:dataAvailable");
    return hasresults_;
  }

  public DataObject getTuple() throws SQLException {
    logger_.debug("ESDBDataSet:getTuple");
    return new ESDBDataTuple(rset_, cacheRS_);
  }

  public void open() throws SQLException {
  }

  public void close() throws SQLException {
    logger_.debug("ESDBDataSet:close in");

    try {
      if (rset_ != null) {
        //Statement statement = rset_.getStatement();
        logger_.debug ("ESDBDataSet:close the resultset");

        if (stat_ instanceof StandardStatement) {
          CoreStatement sstat = (StandardStatement) stat_;
          logger_.debug ("ESDBDataSet:close is instance of CoreStatement");
          if (! sstat.getInCache()) {
            if (rset_ != null) {
              logger_.debug ("ESDBDataSet:close is not in cache "+ stat_.toString());
              rset_.close();
            }

            if (stat_ != null) { 
              logger_.debug ("ESDBDataSet:close the statement 1");
              stat_.close();
            }
          } else {
            logger_.debug ("ESDBDataSet:close open the statement");
            open();
          }
        } else {
          rset_.close();

          if (stat_ != null) { 
            logger_.debug ("ESDBDataSet:close the statement 2 "+stat_.toString());
            stat_.close();
          }
        }

      }
      logger_.debug("ESDBDataSet:close out1");
    } catch (SQLException e) {
      throw e;
    }
    finally {
      if (session_ != null) {
        logger_.debug("ESDBDataSet:close the connection");
        session_.closeConnection(con_, datasourcename_);
      }

      rset_ = null;
      con_ = null;
      idx_ = 0;
      high_ = -1;
      low_ = 0;
      logger_.debug("ESDBDataSet:close out2");
    }
  }

  public void setBoundaries(int low, int high) throws SQLException {
    if (low <= 0) low = 1;
    low_ = low;
    high_ = high;
    while (idx_ < low && next()) ; // Loop until low limit is reached
  }

  public boolean isTransient() {
    return true;
  }

  public ResultSet getResultSet() {
    logger_.debug("ESDBDataSet:getResultSet");
    return rset_;
  }
}

