/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.lite;

/**
 * Class to be used for storing multipart request parameter blocks as
 * well as parameter block headers and parameter block values.
 */
public class DataBlock {
    /** Start position of mBlock within multipart request */
    public int mFrom;
    public int getFrom() {
	return mFrom;
    }
    
    /** End position of mBlock within multipart request */
    public int mTo;
    public int getTo() {
	return mTo;
    }

    /** Block containing parameter block with mName */
    public byte[] mBlock;
    public byte[] getBlock() {
	return mBlock;
    }

    /** Name of parameter block */
    public String mName;
    public String getName() {
	return mName;
    }

    /** States whether block contains file data (true) or not (false) */
    public boolean mIsFileBlock = false;
    public boolean isFileBlock() {
	return mIsFileBlock;
    }
    public void setIsFileBlock(boolean isFB) {
	mIsFileBlock = isFB;
    }

    /** Name of file from/to which block data is downloaded/uploaded */
    public String mFileName = "";
    public String getFileName() {
	return mFileName;
    }
    public void setFileName(String fn) {
	mFileName = (mIsFileBlock) ? fn : "";
    }

    /** Content-type of downloaded/uploaded file, if block contains file data */
    public String mContentType = "";
    public String getContentType() {
	return mContentType;
    }
    public void setContentType(String ct) {
	mContentType = (mIsFileBlock) ? ct : "";
    }

    /**
     * @param from    int containing start position of block within request
     * @param to      int containing end position of block within request
     * @param block   Array of bytes containing block data
     * @param name    String containing name of block
     */
    public DataBlock(int from, int to, byte[] block, String name) {
	mFrom = from;
	mTo = to;
	mBlock = block;
	mName = name;
    }

} // class DataBlock

