/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.openeas.api;

import org.experlog.db.ESDBDataSet;


import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * @author gibello
 */
public class DBDataSet implements DataSet {

  ESDBDataSet ds_ = null;
  
  public DBDataSet(Session s, String sql, String datasource) throws SQLException {
    Connection c = null;
    try {
      c = s.getConnection(datasource);
      Statement st = c.createStatement();
      ResultSet rs = st.executeQuery(sql);
      ds_ = new ESDBDataSet(rs, s.getSession(), s.getConnection(datasource), datasource, st);
    } catch(SQLException e) {
      if(c != null) try { s.closeConnection(c, datasource); } catch(Exception ignore) { }
      ds_ = null;
      throw(e);
    } catch(Exception e) {
      if(c != null) try { s.closeConnection(c, datasource); } catch(Exception ignore) { }
      ds_ = null;
      throw new SQLException(e.getMessage());
    }
  }
  
  public DBDataSet(Session s, String sql) throws SQLException {
    this(s, sql, null);
  }

  /** 
   * Close data set
   * @see org.experlog.openeas.api.DataSet#close()
   */
  public void close() throws SQLException {
    if(ds_ != null) ds_.close();
  }

  /** 
   * @see org.experlog.openeas.api.DataSet#dataAvailable()
   */
  public boolean dataAvailable() throws SQLException {
    return (ds_ == null ? false : ds_.dataAvailable());
  }

  /**
   * @see org.experlog.openeas.api.DataSet#getTuple()
   */
  public DataObject getTuple() throws SQLException {
    return (ds_ == null ? null : ds_.getTuple());
  }

  /**
   * @see org.experlog.openeas.api.DataSet#isTransient()
   */
  public boolean isTransient() {
    return (ds_ == null ? false : ds_.isTransient());
  }

  /**
   * @see org.experlog.openeas.api.DataSet#next()
   */
  public boolean next() throws SQLException {
    return (ds_ == null ? false : ds_.next());
  }

  /**
   * @see org.experlog.openeas.api.DataSet#open()
   */
  public void open() throws SQLException {
    if(ds_ != null) ds_.open();
  }

  /**
   * @see org.experlog.openeas.api.DataSet#setBoundaries(int, int)
   */
  public void setBoundaries(int low, int high) throws SQLException {
    if(ds_ != null) ds_.setBoundaries(low, high);
  }

}
