/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import java.io.File;
import java.sql.SQLException;

import org.experlog.openeas.api.DataSet;
import org.experlog.openeas.api.DataObject;

public class ESFileDataSet implements DataSet {

	private File lst_[];
	private int pos_ = 0;
  int low_ = 0;
  int high_ = -1;

	public ESFileDataSet(File input, String fnm) throws SQLException {

		File tmp = new File(fnm);
		File f;
		if (tmp.isAbsolute()) {
			f = tmp;
		} else {
			String p = input.getParent();
			if (p == null)
				f = tmp;
			else if (p.endsWith(File.separator))
				f = new File(p + tmp);
			else
				f = new File(p + File.separatorChar + tmp);
		}

		if (!f.exists()) {
			throw new SQLException(fnm + ": No such file or directory");
		}
		if (!f.isDirectory()) {
			throw new SQLException(fnm + " is not a valid directory");
		}

		lst_ = f.listFiles();
		if (lst_ == null || lst_.length <= 0) {
			lst_ = null;
		}
	}

	public boolean next() throws SQLException {
		return dataAvailable();
	}

	public boolean dataAvailable() throws SQLException {
		if (lst_ == null || (high_ >= low_ && pos_ >= high_)) return false;
		return (pos_ < lst_.length);
	}

	public DataObject getTuple() throws SQLException {
		if (!dataAvailable()) throw new SQLException("No more data available");
		ESSimpleDataTuple dt = new ESSimpleDataTuple();
		File ff = lst_[pos_++];
		String fnm = ff.getName();
		dt.put("name", fnm);
		int pos = fnm.indexOf('.');
		if (pos >= 0)
			dt.put("ext", fnm.substring(pos + 1));
		else
			dt.put("ext", "");

		dt.put("isfile", (ff.isFile() ? "1" : "0"));
		dt.put("lastmodified", Long.toString(ff.lastModified()));

		return dt;
	}

	public void open() throws SQLException {
	}

	public void close() throws SQLException {
	}

	public boolean isTransient() {
		return true;
	}

	public void setBoundaries(int low, int high) throws SQLException {
		if(low<=0) low = 1;
		low_ = low-1;
		high_ = high;
		while(pos_ < low_ && next()) pos_++; // Skip to low index
  }
}

